
# q-expansion of newform 2184.2.a.c, downloaded from the LMFDB on 12 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2184
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [-2], [1], [-4], [1], [-6], [4], [0], [6], [0], [6], [-6], [4], [0], [-2], [4], [-2], [4], [0], [-6], [8], [12], [10], [-14], [6], [16], [12], [14], [2], [-8], [12], [18], [4], [-10], [8], [-2], [20], [-8], [-2], [20], [-10], [-8], [2], [6], [16], [12], [0], [-4], [-26], [-22], [24], [2], [-28], [-6], [-16], [-2], [0], [-26], [18], [4], [-18], [-12], [-24], [-6], [30], [12], [-14], [12], [14], [18], [16], [-8], [-10], [-4], [16], [-18], [-34], [10], [10], [-20], [38], [-24], [18], [16], [-4], [10], [26], [-10], [32], [12], [-32], [8], [12], [-28], [-8], [-26], [34], [20], [30], [44], [-18], [-36], [10], [-28], [18], [-12], [-14], [-32], [26], [-8], [-26], [2], [-20], [-8], [2], [-28], [-8], [-26], [20], [38], [-30], [6], [12], [-28], [22], [6], [48], [0], [30], [-44], [-16], [-40], [38], [42], [-46], [46], [52], [30], [10], [28], [38], [-32], [-4], [30], [24], [38], [2], [36], [24], [-50], [-38], [12], [-24], [-12], [40], [16], [18], [42], [-10], [-12], [-6], [-24], [36], [10], [-56], [24], [-58], [34], [-26], [60], [-18], [-16], [-6], [40], [50], [60], [54], [-16], [-50], [-32], [28], [22], [42], [-24], [22], [-18], [20], [26], [48], [-46], [-36], [60], [22], [-12], [18], [-14], [-18], [50], [24], [30], [8], [22], [-46], [52], [-50], [-64], [-28], [2], [-28], [-46], [-58], [-32], [36], [8], [26], [-8], [58], [16], [-50], [22], [-8], [74], [-16], [-36], [22], [-30], [40], [-64], [44], [-66], [28], [16], [74], [4], [0], [34], [-10], [-68], [-40], [52], [52], [-16], [-18], [2], [56], [32], [-4], [-52], [24], [62], [-78], [0], [26], [78], [4], [6], [76], [-42], [-54], [-8], [20], [38], [-34], [-46], [28], [30], [-54], [12], [-66], [46], [4], [10], [-24], [34], [8], [-20], [-2], [42], [28], [-32], [8], [8], [-10], [28], [56], [-14], [-42], [-40], [-38], [-26], [-52], [-62], [-12], [-50], [-66], [-72], [-42], [36], [68], [-22], [-18], [-24], [52], [-76], [-78], [12], [46], [-80], [-26], [-16], [-58], [-78], [-20], [-48], [-70], [76], [-24], [18], [66], [-68], [-70], [14], [-32], [-14], [34], [-76], [44], [8], [62], [-18], [22], [88], [44], [-52], [52], [-34], [-30], [-38], [-56], [54], [42], [-66], [48], [78], [36], [70], [12], [-16], [6], [-28], [-38], [-34], [-56], [22], [-30], [48], [52], [50], [16], [70], [18], [56], [-52], [-68], [42], [-42], [-56], [-38], [84], [20], [24], [62], [80], [78], [-84], [-72], [50], [-70], [74], [-10], [-62], [88], [-6], [84], [24], [-16], [22], [28], [32], [50], [-50], [-20], [12], [48], [26], [80], [50], [20], [22], [-50], [-46], [-64], [-6], [14], [-48], [46], [90], [-44], [28], [-46], [30], [-84], [-52], [-86], [-18], [-24], [-64], [2], [24], [-10], [22], [72], [-60], [26], [-42], [52], [66], [28], [24]]
